// Copyright AudioKit. All Rights Reserved. Revision History at http://github.com/AudioKit/AudioKit/
// This file was auto-autogenerated by scripts and templates at http://github.com/AudioKit/AudioKitDevTools/

import AVFoundation

/// AudioKit version of Apple's Decimator Audio Unit
///
public class Decimator: Node {

    fileprivate let effectAU = AVAudioUnitEffect(appleEffect: kAudioUnitSubType_Distortion)

    let input: Node

    /// Connected nodes
    public var connections: [Node] { [input] }

    /// Underlying AVAudioNode
    public var avAudioNode: AVAudioNode { effectAU }

    /// Specification details for decimation
    public static let decimationDef = NodeParameterDef(
        identifier: "decimation",
        name: "Decimation",
        address: AUParameterAddress(kDistortionParam_Decimation),
        defaultValue: 50,
        range: 0 ... 100,
        unit: .percent)

    /// Decimation (Percent) ranges from 0 to 100 (Default: 50)
    @Parameter(decimationDef) public var decimation: AUValue

    /// Specification details for rounding
    public static let roundingDef = NodeParameterDef(
        identifier: "rounding",
        name: "Rounding",
        address: AUParameterAddress(kDistortionParam_Rounding),
        defaultValue: 0,
        range: 0 ... 100,
        unit: .percent)

    /// Rounding (Percent) ranges from 0 to 100 (Default: 0)
    @Parameter(roundingDef) public var rounding: AUValue

    /// Specification details for finalMix
    public static let finalMixDef = NodeParameterDef(
        identifier: "finalMix",
        name: "Final Mix",
        address: AUParameterAddress(kDistortionParam_FinalMix),
        defaultValue: 50,
        range: 0 ... 100,
        unit: .percent)

    /// Final Mix (Percent) ranges from 0 to 100 (Default: 50)
    @Parameter(finalMixDef) public var finalMix: AUValue

    /// Tells whether the node is processing (ie. started, playing, or active)
    public var isStarted = true

    /// Initialize the decimator node
    ///
    /// - parameter input: Input node to process
    /// - parameter decimation: Decimation (Percent) ranges from 0 to 100 (Default: 50)
    /// - parameter rounding: Rounding (Percent) ranges from 0 to 100 (Default: 0)
    /// - parameter finalMix: Final Mix (Percent) ranges from 0 to 100 (Default: 50)
    ///
    public init(
        _ input: Node,
        decimation: AUValue = decimationDef.defaultValue,
        rounding: AUValue = roundingDef.defaultValue,
        finalMix: AUValue = finalMixDef.defaultValue) {
        self.input = input

        associateParams(with: effectAU)

        self.decimation = decimation
        self.rounding = rounding
        self.finalMix = finalMix
        // Since this is the Decimator, mix it to 100% and use the final mix as the mix parameter

        // turn off all the other distortions
        func zero(_ param: AudioUnitParameterID) {
            if let avAudioUnit = avAudioNode as? AVAudioUnit {
                AudioUnitSetParameter(avAudioUnit.audioUnit, param: param, to: 0)
            }
        }

        zero(kDistortionParam_PolynomialMix)
        zero(kDistortionParam_RingModMix)
        zero(kDistortionParam_DelayMix)
    }

    /// Function to start, play, or activate the node, all do the same thing
    public func start() {
        effectAU.bypass = false
        isStarted = true
    }

    /// Function to stop or bypass the node, both are equivalent
    public func stop() {
        effectAU.bypass = true
        isStarted = false
    }
}
